'*** Fishing Mastery Self I - does what it says on the box ***'
'-------------------------------------------------------------'
' - Start near fishing hole(s)...                             '
'   + Make sure you have a license...                         '
'   + And have the fishing pole equipped!                     '
' - Script will wait for Go                                   '
'   + Selects nearest hole when no hole has been selected     '
'   + Active hole can be changed (just press the button)      '
'   + Pause function is available                             '
' - Script will release... (configure this below)             '
'   + Minnows                                                 '
'   + Guppies                                                 '
'   + Mollies                                                 '
' - Script will keep                                          '
'   + anything else, in main pack                             '
' - Script will stop once pack is full or burden reaches max  '
' - Script will log all direct @tells                         '
' - Configure stuff below (Constants and Global Vars)         '
'-------------------------------------------------------------'

'------------------------'
'*** SCRIPT CONSTANTS ***'
'------------------------'
Const PanelName 	= "Fishing Mastery Self I"
Const ScriptVersion 	= "v0.4 alpha"

Const FishingHoleName 	= "Fishing Hole"
Const numFishAttempts 	= 15
Const MaxBurdenPercent 	= 250
Const delayFishing 	= 5500
Const delaySideStep 	= 700

'-------------------'
'*** GLOBAL VARS ***'
'-------------------'
bPaused 		= True
AutoTellReply 		= ""
ReleaseFish 		= Array("*Minnow", "*Guppy", "*Molly")
opmFishing 		= (opmChatWnd Or opmConsole)

'__________Do_not_change_stuff_below_this_line__________
'          (Unless you know what you're doing)

bRunning 		= True
numLastMsg 		= 0
Set acoFishingHole 	= Nothing

'---------------------------------'
'*** MAIN (SCRIPT ENTRY POINT) ***'
'---------------------------------'
Sub Main()

  skapi.OutputLine PanelName & " " & ScriptVersion & " starting up...", _
    opmFishing, cmcYellow
  
  'Init control-panel and other stuff
  InitScript
  
  'Main loop
  While bRunning
    If bPaused Then
      'Just wait for events
      skapi.WaitEvent 5000
    Else
      
      'Try to fish a few times
      skapi.OutputLine "Fishing... (~" & numFishAttempts & " times)", _
        opmFishing, cmcCyan
      For a = 1 to numFishAttempts
        ManageInventory
        If (Not bRunning) Or bPaused Then Exit For
        Fish
      Next
      
      'Release the smaller fish
      If Not bPaused Then
        skapi.OutputLine "Releasing...", opmFishing, cmcCyan
        Release
      End If
      
      'Skip the next part when exiting or paused
      If bRunning And (Not bPaused) Then
        'Salvage junk
        '[...]
      
        'Prevent being logged out by idle timer
        '(and look a bit funny in the process)
        skapi.Keys cmidSideStepLeft, kmoDown
        skapi.WaitEvent delaySideStep, wemFullTimeout
        skapi.Keys cmidSideStepLeft, kmoUp
      End If
      
      'Process events from panel
      skapi.WaitEvent
      
    End If
  Wend
  
  'Clean up before we quit...
  CleanUp
  
End Sub

'------------------------------------'
'*** EVENT HANDLER OBJECT THINGHY ***'
'------------------------------------'
Class EventHandlerThingy
  
  'This handles any control events from the panel
  Public Sub OnControlEvent(szPanel, szControl, dictSzValue)
    'Check which control this belongs to...
    Select Case szControl
      Case "btnQuit"
        'Quit button, so exit...
        skapi.OutputLine "Exiting after completing current cycle...", _
          opmFishing, cmcCyan
        bRunning = False
      Case "btnHole"
        'Select a new fishing hole
        Set acoNewHole = skapi.acoSelected
        If (acoNewHole.szName <> FishingHoleName) Or _
           (acoNewHole.ocm <> ocmNPC) Then
          skapi.OutputLine "Select a Fishing Hole first!", _
            opmFishing, cmcRed
        Else
          Set acoFishingHole = acoNewHole
          skapi.OutputLine "Fishing Hole changed!", opmFishing, cmcCyan
        End If
      Case "btnPause"
        'Pause or unpause the macro
        If bPaused Then
          bPaused = False
          skapi.SetControlProperty PanelName, "btnPause", "text", "Pause"
        Else
          bPaused = True
          skapi.SetControlProperty PanelName, "btnPause", "text", "Resume"
        End If
      Case "lstMsgs"
        'Message list, display selected message...
         skapi.GetControlProperty PanelName, "lstMsgs", _
           "Data(" & dictSzValue("lstMsgs") & ")"
    End Select
  End Sub
  
  'This handles clicking a tell in the list (display it)
  Public Sub OnControlProperty(szPanel, szControl, szProperty, vValue)
    If (szPanel = PanelName) And (szControl = "lstMsgs") And (szProperty = "Data") Then
      skapi.OutputLine "@Tell: " & vValue, opmChatWnd, cmcDarkYellow
    End If
  End Sub
  
  'This handles incoming @tells
  Sub OnTell(acoSender, acoReceiver, szMsg)
    'Log the message in the list...
    LogTell acoSender.szName, szMsg
    'Reply if AutoTellReply is not empty
    If AutoTellReply <> "" Then
      acoSender.Tell AutoTellReply
    End If
  End Sub
  
End Class

'-------------------'
'*** SCRIPT INIT ***'
'-------------------'
Sub InitScript
  strPanel = _
    "<view title='" & PanelName & "' icon='10874' width='200' height='205'>" & _
    "  <control progid='DecalControls.FixedLayout'>" & _
    "    <control name='btnQuit' progid='DecalControls.PushButton' text='Quit' left='10' top='0' width='180' height='20' />" & _
    "    <control name='btnPause' progid='DecalControls.PushButton' text='Go!' left='10' top='25' width='180' height='20' />" & _
    "    <control name='btnHole' progid='DecalControls.PushButton' text='Set Hole' left='10' top='50' width='180' height='20' />" & _
    "    <control name='lstMsgs' progid='DecalControls.List' left='10' top='75' width='180' height='90'>" & _
    "      <column progid='DecalControls.TextColumn' fixedwidth='180' />" & _
    "      <row><data>*** @Tells received ***</data></row>" & _
    "    </control>" & _
    "  </control>" & _
    "</view>"
  skapi.ShowControls strPanel, True
  Set handler = New EventHandlerThingy
  skapi.AddHandler evidOnControlEvent, handler
  skapi.AddHandler evidOnControlProperty, handler
  skapi.AddHandler evidOnTell, handler
End Sub

'---------------'
'*** CLEANUP ***'
'---------------'
Sub CleanUp
  'Remove all handlers...
  skapi.RemoveHandler evidNil, Nothing
  skapi.RemoveControls PanelName
End Sub

'------------------------------------------------'
'*** FINDFISHINGHOLE - Tries to find a nearby ***'
'***   spot to fish, returns false if failed  ***'
'------------------------------------------------'
Function FindFishingHole
  'Find all holes in range
  Set acfHoles = skapi.AcfNew
  acfHoles.szName = FishingHoleName
  acfHoles.ocm = ocmNPC
  'Get the nearest one
  Set acoFishingHole = acfHoles.AcoGetSorted(skapi.SortByDist)
  
  If acoFishingHole Is Nothing Then
    skapi.OutputLine "Can't find a fishing hole! Please start this script near one!", _
      opmFishing, cmcRed
    'Stop the script after completing main loop
    bRunning = False
    FindFishingHole = False
  Else
    FindFishingHole = True
  End If
End Function

'----------------------------------------------'
'*** FISH - Fishes in a nearby fishing hole ***'
'***   Make sure fishing pole is equipped!  ***'
'----------------------------------------------'
Sub Fish
  'Check if we already have a fishing hole
  If acoFishingHole Is Nothing Then
    'See if there are any holes in range
    If Not FindFishingHole Then Exit Sub
  End If
  'Now use it to fish!
  acoFishingHole.Use
  skapi.WaitEvent delayFishing, wemFullTimeout
End Sub

'----------------------------------------------'
'*** RELEASE - Releases the small fish from ***'
'***   the inventory into the fishing hole  ***'
'----------------------------------------------'
Sub Release
  'Check for the fishing hole again...
  If acoFishingHole Is Nothing Then
    'See if there are any holes in range
    If Not FindFishingHole Then Exit Sub
  End If
  'See if there's anything in the inventory to be released
  For a = 0 to UBound(ReleaseFish)
    Set acoReleaseMe = skapi.AcoFindInInv(ReleaseFish(a))
    While (Not (acoReleaseMe Is Nothing)) And (Not bPaused)
      skapi.OutputLine " + " & acoReleaseMe.szName, opmFishing, cmcLightBlue
      'Check if it is stacked
      If acoReleaseMe.citemStack > 1 Then
        skapi.OutputLine "   * ITEM IS STACKED! TRYING TO SPLIT...", opmFishing, cmcRed
        skapi.SelectAco acoReleaseMe
        skapi.Keys cmidSplitSelected & "1{return}"
        skapi.WaitEvent 100, wemFullTimeout
      End If
      'Release the fish
      acoReleaseMe.MoveToStack(acoFishingHole)
      skapi.WaitEvent 1500, wemFullTimeout
      Set acoReleaseMe = skapi.AcoFindInInv(ReleaseFish(a))
    Wend
  Next
End Sub

'---------------------------------------------------'
'*** LOGTELL - logs an @tell to the list control ***'
'---------------------------------------------------'
Sub LogTell(szName, szMsg)
  skapi.OutputLine "@tell [" & szMsg & "] from [" & szName & "]", _
    (opmFishing And Not opmChatWnd), cmcDarkYellow
  numLastMsg = numLastMsg + 1
  skapi.GetControlProperty PanelName, "lstMsgs", "AddRow"
  skapi.SetControlProperty PanelName, "lstMsgs", "Data(0, " & numLastMsg & ")", _
    szName & " [" & szMsg & "]"
End Sub

'------------------------------------------------------'
'*** MANAGEINVENTORY - Stacks all stackable items,  ***'
'***   checks available pack space and burden level ***'
'------------------------------------------------------'
Sub ManageInventory
  'Defragment main pack
  '[...]

  'Check available pack space
  If skapi.acoChar.citemmax - skapi.acoChar.cItemContents < 1 Then
    skapi.OutputLine "Not enough pack space! Stopping...", _
      opmFishing, cmcRed
    bRunning = False
  End If
  
  'Check burden level
  If skapi.BurdenCur / skapi.BurdenFull * 100 > MaxBurdenPercent Then
    skapi.OutputLine "Burden too high! Stopping...", _
      opmFishing, cmcRed
    bRunning = False
  End If
End Sub

'-------------------------------'
'*** THE END - for now... :) ***'
'-------------------------------'
